#-----------------------------------------------------------------------------
# Name:        ZRTChat.py
# Purpose:     Realtime Chat for Zope
#
# Author:      Jim Washington (jwashin@vt.edu)
#
# Created:     15 July 2001
# RCS-ID:      $Id: ZRTChat.py $
# Copyright:   (c) 1999-2001 Jim Washington
# License:     MIT License
#-----------------------------------------------------------------------------

__doc__ = """ZRTChat Product.  Real-time Chat for Zope."""

__version__ = '0.4.0'

from Globals import DTMLFile
from Globals import Persistent
from Globals import InitializeClass
from OFS.Folder import Folder
from Products.PythonScripts import PythonScript
from App.FindHomes import INSTANCE_HOME, SOFTWARE_HOME
import os
import string
# To make ZRTChats good travelers in the Zope realm, we will try our hand at
# interfaces.  A Good Idea. The New Religion(TM).
try:
    from chat import ChatManager

except:
    class ChatManager:
        pass

#get the data class so we can add them.
import ZRTChatItem

#Constructor interface or ZMI
manage_addZRTChatForm = DTMLFile('www/ZRTChatAdd', globals())

def manage_addZRTChat(self, id, REQUEST=None):
    """Add a ZRTChat to a folder."""
    self._setObject(id, ZRTChat(id, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

#this is the folderish class that manages ZODB-based chats
class ZRTChat(Folder):
    """ZRTChat folder."""

    __implements__ = (ChatManager,)

    meta_type = 'ZRTChat' # add-list name

#OK, we will allow copy and paste, but I will not be responsible
#   dontAllowCopyAndPaste = 0
    manage_options = Folder.manage_options[:2] + ({'label':'Chat Properties', 'action':'manage_chatPropertiesForm'},) + Folder.manage_options[2:]

#it turns out that the permissions are OK from Folder.  Inherit __ac_permissions__ by doing nothing!

    def __init__(self, id, REQUEST):
        """initialize a new instance of ZRTChat"""
        self.id = id
        self.bgcolor = '#ffffff'
        self.title = "Nice weather we are having..."
        self.returnlink = ''
        self.returntext = ''
        self.homelink = ''
        self.hometext = ''
        self.starttext = """First, identify yourself.
Type your name at the bottom of
the page and press "Submit name."
A new form will appear in the same window.
Type your message in the text box
 and press "Post message." """
        self.toptext = "How is the weather where you are?"
        if REQUEST is not None:
          if REQUEST.has_key('addDTML'):
            self.add_default_DTML(REQUEST)

    def add_default_DTML(self, REQUEST):
        #find the software location
        for k in [INSTANCE_HOME, SOFTWARE_HOME]:
            location = os.path.join(k,'Products','ZRTChat','www')
            if os.path.exists(location):
                filesdir = location
        for k in [['index_html','Chat Frameset'],['tablerowitem','Display item in table'],['topframe','Title and topic'],
        ['returnlinks','Links in the topic frame'],['controls','List of controls'],['chatframe','The chats are displayed here'],
        ['countform','Control for setting how many chats to see'],['latestlastform','Control for reversing ordering of the chats'],
        ['refreshform','Control to set the refresh frequency'],['formframe','Chat user interface'],['noframes','Display chat with no frames'],
        ['notablerowitem', 'Display a chat item without table for noframes'],
        ['cssrowitem', 'Display a chat item css formatted'],['style', 'Inline stylesheet'],
        ['standard_html_header', 'Standard HTML Header'],
        ['standard_html_footer', 'Standard HTML Footer']]:

            t = open(os.path.join(filesdir,k[0])+'.dtml')
            d = t.readlines()
            t.close()
            self.manage_addDTMLMethod(k[0])
            self[k[0]].manage_edit(title = k[1], data=string.join(d))
        #pythonscript for the frameset. Easier this way...
        PythonScript.manage_addPythonScript(self, 'frameset')
        t = open(os.path.join(filesdir,'frameset.py'))
        d = t.readlines()
        t.close()
        self.frameset.ZPythonScript_setTitle('The calculated frameset')
        self.frameset.ZPythonScript_edit(params='', body=string.join(d,'\n'))


    def topic(self):
        """return the topic"""
        return self.title


# DTML routines for the product
# the frameset
#    index_html = DTMLFile('www/index_html', globals())
# the frame where all the chats go
#    chatframe = DTMLFile('www/chatframe', globals())
#    tablerowitem= DTMLFile('www/tablerowitem', globals())
# the top frame, with the title and topic
#    topframe= DTMLFile('www/topframe', globals())
#    returnlinks= DTMLFile('www/returnlinks', globals())
# the upper right frame with the controls
#    controls= DTMLFile('www/controls', globals())
#    countform = DTMLFile('www/countform', globals())
#    latestlastform= DTMLFile('www/latestlastform', globals())
#    refreshform= DTMLFile('www/refreshform', globals())
#the frame at the bottom that handles most of the user interface
#    formframe= DTMLFile('www/formframe', globals())
#DTML for noframes
#    noframes= DTMLFile('www/noframes', globals())
#    notablerowitem= DTMLFile('www/notablerowitem', globals())
#    style = DTMLFile('www/style', globals())

# Management Interface for the chat properties
    manage_chatPropertiesForm = DTMLFile('www/ZRTChatEdit', globals())

    def getChatItems(self, REQUEST=None, itemCount=25, ordering='latest_first'):
        """Return the list of chat items for display"""
        #first, assure the location exists for the items.
        if not 'Chat_Items' in self.objectIds('Folder'):
            self.manage_addFolder('Chat_Items','Chat Items')
        if REQUEST is not None:
            if REQUEST.cookies.has_key('itemCount'):
                itemCount = int(REQUEST.cookies['itemCount'])
            if REQUEST.cookies.has_key('displayOrder'):
                ordering = REQUEST.cookies['displayOrder']
        templist = self['Chat_Items'].objectValues('ChatItem')
        if itemCount == 0:
            theList = templist
        else:
            itemCount = min(itemCount,len(templist))
            theList = templist[-itemCount:]
        if ordering == 'latest_first':
            theList.reverse()
        return theList

    def setTopic(self,aTopic,REQUEST=None):
        """Set the topic"""
        self._topic = aTopic

    def manage_edit(self, REQUEST=None):
        """set the new properties, but only if actually changed"""
        if REQUEST is not None:
            self.title = REQUEST.title
            self.starttext = REQUEST.starttext
            self.returnlink = REQUEST.returnlink
            self.returntext = REQUEST.returntext
            self.homelink = REQUEST.homelink
            self.hometext = REQUEST.hometext
            self.toptext = REQUEST.toptext
            self.bgcolor = REQUEST.bgcolor
           
            return self.manage_propertiesForm()

    def addChatItem(self,REQUEST):
        """Add a chat item"""
        ZRTChatItem.manage_addChatItem(self['Chat_Items'],REQUEST)

    #remove chatItems if necessary, but no code necessary since we are using a standard folder
    #delChatItems = Folder.manage_delObjects
InitializeClass(ZRTChat)
